/*
 * Copyright (C) 2010 - Paulo Ricardo Zanoni <pzanoni@mandriva.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

#include <CrtcInfo.h>

#include <cstdlib>
#include <iostream>
using namespace std;

CrtcInfo::CrtcInfo(Display *display,
		   XRRScreenResources *res,
		   RRCrtc x_crtc)
{

    xid = x_crtc;

    update_info(display, res);
}

void CrtcInfo::update_info(Display *display, XRRScreenResources *res)
{
    int i;
    XRRCrtcInfo *crtc_info;

    if (xid == None) {
	cerr << "Error: trying to update_info() from crtc None\n";
	exit(1);
    }

    crtc_info = XRRGetCrtcInfo(display, res, xid);

    x = crtc_info->x;
    y = crtc_info->y;
    width = crtc_info->width;
    height = crtc_info->height;
    mode = crtc_info->mode;
    timestamp = crtc_info->timestamp;
    rotation = crtc_info->rotation;

    outputs.clear();
    for (i = 0; i < crtc_info->noutput; i++) {
	outputs.push_back(crtc_info->outputs[i]);
    }

    supported_rotations = crtc_info->rotations;

    XRRFreeCrtcInfo(crtc_info);
}

bool CrtcInfo::rotation_is_supported(Rotation rotation)
{
    //cout << "rotation: " << rotation <<  ", supported: " << supported_rotations
	 //<< "\n";
    return (supported_rotations & rotation);
}
