/*
 * Copyright (C) 2010 - Paulo Ricardo Zanoni <pzanoni@mandriva.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

#ifndef OUTPUTWIDGET_H
#define OUTPUTWIDGET_H

#include <QtGui>
#include <QWidget>

#include "RandrInfo.h"

class OutputWidget : public QWidget
{
    Q_OBJECT

    public:
        OutputWidget(RandrInfo *randr_info_arg,
		     RROutput output_id_arg,
		     QWidget *parent = 0);
	~OutputWidget();

	RRMode   get_selected_mode();
	Rotation get_selected_rotation();
	void     set_selected_mode(RRMode mode_id);
	void     set_selected_rotation(Rotation rotation);
	RROutput get_output();
	void     update_info();

    signals:
	void output_changed(RROutput output, RRMode mode, Rotation rotation);

    private slots:
        void mode_combo_changed(int index);
        void rotation_combo_changed(int index);

    private:
	RandrInfo *randr_info;
	RROutput output_id;
	RRMode selected_mode;
	Rotation selected_rotation;

	QComboBox *mode_combo;
	QComboBox *rotation_combo;

	// When we're doing update_info() we remove and then readd all the
	// combobox items. I know this is not cool. FIXME.
	// But the problem is that when we're readding stuff the combo keeps
	// emitting its change signals, so we need to have some way to stop
	// propagating these events to the upper layers
	bool dont_emit_changes;
};

#endif
