/*
 * Copyright (C) 2010 - Paulo Ricardo Zanoni <pzanoni@mandriva.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

#ifndef RANDRINFO_H
#define RANDRINFO_H

#include <map>

#include <X11/Xlib.h>
#include <X11/extensions/Xrandr.h>

#include "OutputInfo.h"
#include "ModeInfo.h"
#include "CrtcInfo.h"

class RandrInfo
{
    public:
	RandrInfo(Display *display, int screen_nbr);
	~RandrInfo();
	void update_info();

	std::list<RROutput> get_outputs();
	std::list<RRCrtc>   get_crtcs();

	std::string get_output_name(RROutput output_id);
	std::string get_mode_name(RRMode mode_id);

	RROutput get_output_from_name(std::string name);
	RRMode   get_mode_from_name(std::string name);

	RRMode            get_output_mode(RROutput output_id);
	Rotation          get_output_rotation(RROutput output_id);
	std::list<RRMode> get_output_modes(RROutput output_id);
	RRCrtc            get_output_crtc(RROutput output_id);
	bool              output_is_connected(RROutput output_id);

	enum SetConfigStatus {
	    SetConfigSuccess,
	    ScreenSizeError,
	    SetCrtcConfigError,
	    RotationError
	};
	typedef std::map<RROutput, RRMode>   OutputToModeMap;
	typedef std::map<RROutput, Rotation> OutputToRotationMap;

	SetConfigStatus set_config(OutputToModeMap output_to_mode,
				   OutputToRotationMap output_to_rotation,
				   bool extend_desktop,
				   std::list<RROutput> output_layout);

	bool desktop_is_extended();

	void register_for_events(Window win);
	void process_event (XEvent *ev);

	std::list<RROutput> get_outputs_in_order();

	int wanted_x11_event;

    private:
	typedef std::map<RROutput, OutputInfo*> OutputMap;
	typedef std::map<RRMode, ModeInfo*> ModeMap;
	typedef std::map<RRCrtc, CrtcInfo*> CrtcMap;
	OutputMap outputs;
	ModeMap modes;
	CrtcMap crtcs;

	Display *display;
	int screen_nbr;
	XRRScreenResources *res;
	int xrr_event_base, xrr_error_base;
	Window event_window;


	enum ScreenInfo {
	    Width,
	    Height
	};
	unsigned int get_screen_info(ScreenInfo info);
	void set_screen_size(unsigned int width, unsigned int height);
	bool last_active_crtc(RRCrtc crtc);
	bool crtc_supports_rotation(RRCrtc crtc, Rotation rotation);

	void hotplug_output(RROutput output, Connection connection);
	void real_process_event (XEvent *ev);
};

#endif
