/*
 * Copyright (C) 2010 - Paulo Ricardo Zanoni <pzanoni@mandriva.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301,
 * USA.
 */

#include <cstdlib>
#include <iostream>

#include <QApplication>
#include <QTranslator>

#include "YargWindow.h"

// For locking/open:
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

YargWindow *main_window;

void sig_usr1(int sig_number) {

    std::cout << "SIGUSR1 received\n";
    if (main_window->isHidden()) {
	main_window->show();
    }
}


int main(int argc, char *argv[])
{
    int fd, lock_ret;
    struct flock lock_s;
    struct sigaction sig_s;
    std::string lock_path;
    lock_path = (QDir::homePath() + "/.yarg.lock").toStdString();
    bool do_show = false;

    for(int i = 0; i < argc; i++) {
	if (strcmp(argv[i], "--show") == 0) {
	    do_show = true;
	}
	if ((strcmp(argv[i], "--help") == 0) || (strcmp(argv[i], "-h") == 0)) {
	    std::cout << "Options:\n";
	    std::cout << "    --show: show the main window when started\n";
	    std::cout << "    --help: show this message\n";
	    std::cout << "For more information, please check the man page\n";
	    return 1;
	}
    }


    fd = open(lock_path.c_str(), O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
    if (fd == -1) {
	std::cerr << "Error: can't open/create " << lock_path << "\n";
	return 1;
    }

    lock_s.l_type = F_WRLCK;
    lock_s.l_whence = SEEK_SET;
    lock_s.l_start = 0;
    lock_s.l_len = 0;
    lock_s.l_pid = 0;

    fcntl(fd, F_GETLK, &lock_s);

    if (lock_s.l_type == F_UNLCK) {
	lock_s.l_type = F_WRLCK;
	lock_ret = fcntl(fd, F_SETLK, &lock_s);
	if (lock_ret != 0) {
	    std::cerr << "Can't lock file lock.\n";
	    return 1;
	}
    } else {
	std::cerr << "Lock file is locked, another instance of yarg is already "
		     "running.\n";
	if (do_show) {
	    std::cerr << "Sending SIGUSR1 to PID " << lock_s.l_pid
		      << " to make it appear.\n";
	    if (kill(lock_s.l_pid, SIGUSR1) != 0) {
		std::cerr << "Error sending SIGUSR1.\n";
	    }
	}
	return 1;
    }

    // Now start
    QApplication app(argc, argv);

    QTranslator qtTranslator;
    QTranslator myappTranslator;

    QDir::setSearchPaths("icons", QStringList(DATA_PREFIX));

    qDebug() << "Translation: " << QLocale::system().name();

    qtTranslator.load("qt_" + QLocale::system().name(),
		      QLibraryInfo::location(QLibraryInfo::TranslationsPath));
    app.installTranslator(&qtTranslator);
    myappTranslator.load("yarg_" + QLocale::system().name(), TRANSLATIONS_PREFIX);
    app.installTranslator(&myappTranslator);

    main_window = new YargWindow();

    // Set the trap for SIGUSR1
    sig_s.sa_handler = &sig_usr1;
    sigemptyset(&sig_s.sa_mask);
    sig_s.sa_flags = 0;
    if (sigaction(SIGUSR1, &sig_s, NULL) != 0) {
	std::cerr << "Error setting action for SIGUSR1!\n";
	return 1;
    }

    if (!QSystemTrayIcon::isSystemTrayAvailable() || do_show) {
	main_window->show();
    }

    app.exec();

    std::cout << "Unlocking...\n";
    lock_s.l_type = F_UNLCK;
    fcntl(fd, F_SETLK, &lock_s);
    close(fd);
    unlink(lock_path.c_str()); // FIXME: unlink when we're killed too?
    return 0;
}
